@@ -12,12 +12,16 @@ class RecipeOption:
       self,
       documentation: str = "",
       valid_values: Sequence[str] = (),
+      deprecated: Optional[str] = "",
   ):
     self.documentation = documentation
 
     self.valid_values = valid_values
     """Possible values. If this is empty, it means no constraint on values.
     """
+    self.deprecated = deprecated
+    """Reason for the option to be deprected, if it is deprecated.
+    """
 
 
 class Recipe:
@@ -63,6 +67,8 @@ Recipe(
     options={
         'url':
         RecipeOption('Url or path of the input template', ),
+        'inline':
+        RecipeOption('Inline input template', ),
         'output':
         RecipeOption('Path of the output', ),
         'md5sum':
@@ -80,14 +86,21 @@ Recipe(
     description=
     'Template recipe which supports remote resource and templating with [jinja2](https://jinja.palletsprojects.com/en/2.10.x/)',
     url='https://pypi.org/project/slapos.recipe.template/',
-    required_options=('template', 'rendered'),
+    required_options=('url', 'output'),
     options={
+        'url':
+        RecipeOption('Url or path of the input template', ),
+        'inline':
+        RecipeOption('Inline input template', ),
+        'output':
+        RecipeOption('Path of the output', ),
         'template':
         RecipeOption(
             'Template url/path, as accepted by `zc.buildout.download.Download.__call__`. For very short template, it can make sense to put it directly into buildout.cfg: the value is the template itself, prefixed by the string `inline:` + an optional newline.',
-        ),
+            deprecated="Use `url` or `inline` options instead"),
         'rendered':
-        RecipeOption('Where rendered template should be stored.', ),
+        RecipeOption('Where rendered template should be stored.',
+                     deprecated="Use `output` option instead"),
         'context':
         RecipeOption(
             """
@@ -98,10 +111,10 @@ Recipe(
  Available types are described below. name is the variable name to declare. Expression semantic varies depending on the type.
 
 Available types:
-	* `raw`: Immediate literal string.
-	* `key`: Indirect literal string.
-	* `import`: Import a python module.
-	* `section`: Make a whole buildout section available to template, as a dictionary.
+  * `raw`: Immediate literal string.
+  * `key`: Indirect literal string.
+  * `import`: Import a python module.
+  * `section`: Make a whole buildout section available to template, as a dictionary.
 
 Indirection targets are specified as `[section]:key` . It is possible to use buildout’s built-in variable replacement instead instead of `key` type, but keep in mind that different lines are different variables for this recipe. It might be what you want (factorising context chunk declarations), otherwise you should use indirect types
 """, ),
@@ -130,11 +143,11 @@ Indirection targets are specified as `[section]:key` . It is possible to use bui
 
 Available types:
 
-	* `rawfile`: Literal path of a file.
-	* `file`: Indirect path of a file.
-	* `rawfolder`: Literal path of a folder. Any file in such folder can be imported.
-	* `folder`: Indirect path of a folder. Any file in such folder can be imported.
-	* `encoding`: Encoding for input template and output file. Defaults to `utf-8`.
+  * `rawfile`: Literal path of a file.
+  * `file`: Indirect path of a file.
+  * `rawfolder`: Literal path of a folder. Any file in such folder can be imported.
+  * `folder`: Indirect path of a folder. Any file in such folder can be imported.
+  * `encoding`: Encoding for input template and output file. Defaults to `utf-8`.
 """, ),
     },
 )
