@@ -26,6 +26,7 @@ from pygls.lsp.types import (
     Command,
     CompletionItem,
     CompletionItemKind,
+    CompletionItemTag,
     CompletionOptions,
     CompletionParams,
     DidChangeTextDocumentParams,
@@ -49,10 +50,23 @@ from pygls.lsp.types.window import ShowDocumentParams
 from pygls.server import LanguageServer
 from pygls.workspace import Document
 
-from . import buildout, code_actions, commands, diagnostic, recipes, types, md5sum, utils
+from . import (
+    buildout,
+    code_actions,
+    commands,
+    diagnostic,
+    md5sum,
+    profiling,
+    recipes,
+    types,
+    utils,
+)
 
 server = LanguageServer()
 
+server.command(commands.COMMAND_START_PROFILING)(profiling.start_profiling)
+server.command(commands.COMMAND_STOP_PROFILING)(profiling.stop_profiling)
+
 reference_start = '${'
 reference_re = re.compile(
     r'\${(?P<section>[-a-zA-Z0-9 ._]*):(?P<option>[-a-zA-Z0-9 ._]+)}')
@@ -100,7 +114,6 @@ async def command_update_md5sum(
   await md5sum.update_md5sum(ls, args[0])
 
 
-@utils.singleton_task
 @server.feature(
     CODE_ACTION,
     CodeActionOptions(resolve_provider=False,
@@ -108,6 +121,7 @@ async def command_update_md5sum(
                           CodeActionKind.QuickFix,
                       ]),
 )
+@utils.singleton_task
 async def lsp_code_action(
     ls: LanguageServer,
     params: CodeActionParams) -> Optional[List[Union[Command, CodeAction]]]:
@@ -140,8 +154,8 @@ async def did_change_watched_file(
     buildout.clearCache(change.uri)
 
 
-@utils.singleton_task
 @server.feature(DOCUMENT_SYMBOL)
+@utils.singleton_task
 async def lsp_symbols(
     ls: LanguageServer,
     params: DocumentSymbolParams,
@@ -202,8 +216,8 @@ async def lsp_symbols(
   return symbols
 
 
-@utils.singleton_task
 @server.feature(COMPLETION, CompletionOptions(trigger_characters=["{", ":"]))
+@utils.singleton_task
 async def lsp_completion(
     ls: LanguageServer,
     params: CompletionParams,
@@ -250,7 +264,7 @@ async def lsp_completion(
     """Calculate the edition to insert option} a ${section:option}
     """
     words_split = re.compile(
-        r'(?P<section>\${[-a-zA-Z0-9 ._]*\:)(?P<option>[-a-zA-Z0-9._]*\}{0,1})'
+        r'(?P<section>\${[-a-zA-Z0-9 ._]*\:)(?P<option>[ ]*[-a-zA-Z0-9._]*\}{0,1})'
     )
     line = doc.lines[pos.line]
     index = 0
@@ -436,17 +450,22 @@ async def lsp_completion(
         if recipe:
           for k, v in recipe.options.items():
             items.append(
-                CompletionItem(label=k,
-                               text_edit=getDefaultTextEdit(
-                                   doc,
-                                   params.position,
-                                   k + ' = ',
-                               ),
-                               kind=CompletionItemKind.Variable,
-                               documentation=MarkupContent(
-                                   kind=MarkupKind.Markdown,
-                                   value=v.documentation,
-                               )))
+                CompletionItem(
+                    label=k,
+                    text_edit=getDefaultTextEdit(
+                        doc,
+                        params.position,
+                        k + ' = ',
+                    ),
+                    kind=CompletionItemKind.Variable,
+                    tags=([CompletionItemTag.Deprecated]
+                          if v.deprecated else []),
+                    documentation=MarkupContent(
+                        kind=MarkupKind.Markdown,
+                        value=
+                        (f'**Deprecated**\n{v.deprecated}\n\n----\n{v.documentation}'
+                         if v.deprecated else v.documentation),
+                    )))
         else:
           # section has no recipe, complete `recipe` as an option name
           items.append(
@@ -530,8 +549,8 @@ async def lsp_completion(
   return items
 
 
-@utils.singleton_task
 @server.feature(DEFINITION)
+@utils.singleton_task
 async def lsp_definition(
     ls: LanguageServer,
     params: TextDocumentPositionParams,
@@ -573,8 +592,8 @@ async def lsp_definition(
   return locations
 
 
-@utils.singleton_task
 @server.feature(REFERENCES)
+@utils.singleton_task
 async def lsp_references(
     server: LanguageServer,
     params: TextDocumentPositionParams,
@@ -635,8 +654,8 @@ async def lsp_references(
   return references
 
 
-@utils.singleton_task
 @server.feature(HOVER)
+@utils.singleton_task
 async def lsp_hover(
     ls: LanguageServer,
     params: TextDocumentPositionParams,
@@ -659,8 +678,8 @@ async def lsp_hover(
   return Hover(contents=f'```\n{hover_text}\n```')
 
 
-@utils.singleton_task
 @server.feature(DOCUMENT_LINK)
+@utils.singleton_task
 async def lsp_document_link(
     ls: LanguageServer,
     params: DocumentLinkParams,
